module Main where

import Control.Monad
import Control.Concurrent (threadDelay)
import Data.IORef
import Data.Maybe

import System.Random

import qualified Graphics.UI.Threepenny as UI
import Graphics.UI.Threepenny.Core

import Minesweeper

main :: IO ()
main = do
    startGUI defaultConfig {jsStatic = Just "view"} setup

setup :: Window -> UI ()
setup w = void $ do
    return w # set title "Minesweeper"

    UI.addStyleSheet w "bootstrap.min.css"
    UI.addStyleSheet w "minesweeper.css"
    rng <- liftIO newStdGen
    
    let b = createBoard 20 0.08 rng :: Board
    liftIO $ putStrLn $ printBoard b
    iob <- liftIO $ newIORef b

    getBody w #+ [
        UI.div #. "container" #+ [
            UI.h1 #+ [string "Minesweeper"],
            UI.h4 #+ [string "Jack Harley jharley@tcd.ie"],
            UI.div #. "row" #+ [
                UI.div # set UI.id_ "gameCont" #+ [mkElement "table" # set UI.id_ "table" #+ rows iob b 0],
                UI.div # set UI.id_ "infoCont" #+ [
                    UI.p #+ [string "Instructions: Click on a square to uncover it. Right click a square to flag it."],
                    UI.p #+ [string "Flagged squares will turn yellow. If you hit a mine all mines will instantly be revealed as red squares."],
                    UI.p #+ [string "You win the game once you have uncovered all squares that do not have mines. If this occurs, the entire board will turn green (except the bomb) to indicate your win!"],
                    UI.p #+ [string "At any time, you can refresh the page to start a new game."],
                    UI.p #+ [string "Good luck!"]
                ] 
            ]
        ],

        -- include custom JS at end of body to stop right clicks opening a menu
        mkElement "script" # set (attr "src") "/static/custom.js"]

    where
        rows iob b r | r < size b = (mkElement "tr" #+ cells iob b r 0) : rows iob b (r+1)
                     | otherwise = []

        cells iob b r c | c < size b = cell iob b (r,c) : cells iob b r (c+1)
                        | otherwise = []
        
        cell iob b (r,c) = do
            cell <- mkElement "td" #. (squareBgColour b (r,c) ++ " " ++ squareTextColour b (r,c)) 
                                   #+ [string $ squareAscii b (r,c)]

            on UI.click cell $ \_ -> do
                liftIO $ putStrLn $ "Click (" ++ show r ++ "," ++ show c ++ ")"
                liftIO $ modifyIORef' iob $ \oldB -> uncover oldB (r,c)
                refresh iob

            on UI.contextmenu cell $ \_ -> do
                liftIO $ putStrLn $ "Right Click (" ++ show r ++ "," ++ show c ++ ")"
                liftIO $ modifyIORef' iob $ \oldB -> flag oldB (r,c)
                refresh iob

            return cell

        -- refresh the board on screen (rerender)
        refresh iob = do
            b <- liftIO $ readIORef iob
                
            table <- getElementById w "table"
            let table' = fromJust table
                
            cont <- getElementById w "gameCont"
            let cont' = return $ fromJust cont
            cont' #+ [mkElement "table" # set UI.id_ "table" #+ rows iob b 0]

            delete table'